if [ ! -r dev/pilot ]; then # pick a default
  ( cd dev ; ln -sf ttyS0 pilot )
fi
( cd usr/lib ; rm -rf libpisock.so )
( cd usr/lib ; ln -sf libpisock.so.9.0.2 libpisock.so )
( cd usr/lib ; rm -rf libpisync.so.1 )
( cd usr/lib ; ln -sf libpisync.so.1.0.3 libpisync.so.1 )
( cd usr/lib ; rm -rf libpisock.so.9 )
( cd usr/lib ; ln -sf libpisock.so.9.0.2 libpisock.so.9 )
( cd usr/lib ; rm -rf libpisync.so )
( cd usr/lib ; ln -sf libpisync.so.1.0.3 libpisync.so )
( cd usr/lib ; rm -rf libpitcl.so )
( cd usr/lib ; ln -sf libpitcl.so.0.0.0 libpitcl.so )
( cd usr/lib ; rm -rf libpitcl.so.0 )
( cd usr/lib ; ln -sf libpitcl.so.0.0.0 libpitcl.so.0 )
